# test program for wregistry gem
# Last modified:  02-Aug-2016

require 'xregistry'
require 'test/unit'

class TestXregistry < Test::Unit::TestCase
  @@app_key    = 'testxreg'
  @@vendor_key = 'Wikareia'

  def setup
    @@reg ||= Registry.new(@@app_key, @@vendor_key)
  end

  def test_new
    assert_equal(Registry, @@reg.class)
  rescue
    flunk "error encountered opening database: #{$!}"
  end

  # test of each_key
  def test_each_key
    keys = []
    @@reg.each_key { |k| keys << k }
    keys.each { |k| @@reg.deleteSection(k) }
    @@reg.writeIntEntry('Section1', 'key', 1)
    @@reg.writeIntEntry('Section2', 'key', 2)
    @@reg.writeIntEntry('Section3', 'key', 3)
    value = []
    @@reg.each_key { |k| value << k }
    assert_equal(%w[Section1 Section2 Section3], value)
  end

  # test of find
  def test_find
    @@reg.writeIntEntry('Section', 'key1', 1)
    @@reg.writeIntEntry('Section', 'key2', 2)
    @@reg.writeIntEntry('Section', 'key3', 3)
    value = []
    section = @@reg.find('Section')
    section.each_key { |k| value << [k, section.find(k)] }
    assert_equal([['key1', 1], ['key2', 2], ['key3', 3]], value)
  end

  # test of readBoolEntry
  def test_read_bool_entry
    value = @@reg.readBoolEntry('Settings', 'bool_key1')
    assert_equal(false, value)

    value = @@reg.readBoolEntry('Settings', 'bool_key1', true)
    assert_equal(true, value)

    value = @@reg.readBoolEntry('Settings', 'bool_key1', false)
    assert_equal(false, value)

    @@reg.writeIntEntry('Settings', 'bool_key1', 2)
    value = @@reg.readBoolEntry('Settings', 'bool_key1')
    assert_equal(true, value)

    @@reg.writeRealEntry('Settings', 'bool_key1', 2.5)
    assert_raise(TypeError) { @@reg.readBoolEntry('Settings', 'bool_key1') }

    @@reg.writeStringEntry('Settings', 'bool_key1', '2')
    assert_raise(TypeError) { @@reg.readBoolEntry('Settings', 'bool_key1') }
  end

  # test of readIntEntry
  def test_read_int_entry
    value = @@reg.readIntEntry('Settings', 'int_key1')
    assert_equal(0, value)

    value = @@reg.readIntEntry('Settings', 'int_key1', 123)
    assert_equal(123, value)

    @@reg.writeBoolEntry('Settings', 'int_key1', true)
    value = @@reg.readIntEntry('Settings', 'int_key1')
    assert_equal(1, value)

    @@reg.writeRealEntry('Settings', 'int_key1', 2.5)
    value = @@reg.readIntEntry('Settings', 'int_key1')
    assert_equal(2, value)

    @@reg.writeStringEntry('Settings', 'int_key1', '2')
    value = @@reg.readIntEntry('Settings', 'int_key1')
    assert_equal(2, value)

    @@reg.writeStringEntry('Settings', 'int_key1', 'abc')
    value = @@reg.readIntEntry('Settings', 'int_key1')
    assert_equal(0, value)
  end

  # test of readRealEntry
  def test_read_real_entry
    value = @@reg.readRealEntry('Settings', 'real_key1')
    assert_equal(0.0, value)

    value = @@reg.readRealEntry('Settings', 'real_key1', 12.5)
    assert_equal(12.5, value)

    @@reg.writeBoolEntry('Settings', 'real_key1', true)
    value = @@reg.readRealEntry('Settings', 'real_key1')
    assert_equal(1.0, value)

    @@reg.writeIntEntry('Settings', 'real_key1', 2)
    value = @@reg.readRealEntry('Settings', 'real_key1')
    assert_equal(2.0, value)

    @@reg.writeStringEntry('Settings', 'real_key1', '2.5')
    value = @@reg.readRealEntry('Settings', 'real_key1')
    assert_equal(2.5, value)

    @@reg.writeStringEntry('Settings', 'real_key1', 'abc')
    value = @@reg.readRealEntry('Settings', 'real_key1')
    assert_equal(0, value)
  end

  # test of readStringEntry
  def test_read_string_entry
    string = @@reg.readStringEntry('Settings', 'string_key1')
    assert_equal('', string)

    string = @@reg.readStringEntry('Settings', 'string_key1', 'default value')
    assert_match(/ value/, string)

    @@reg.writeBoolEntry('Settings', 'string_key1', true)
    value = @@reg.readStringEntry('Settings', 'string_key1')
    assert_equal('1', value)

    @@reg.writeIntEntry('Settings', 'string_key1', 2)
    value = @@reg.readStringEntry('Settings', 'string_key1')
    assert_equal('2', value)

    @@reg.writeRealEntry('Settings', 'string_key1', 2.5)
    value = @@reg.readStringEntry('Settings', 'string_key1')
    assert_equal('2.5', value)
  end

  # test of writeBoolEntry
  def test_write_bool_entry
    @@reg.writeBoolEntry('Settings', 'bool_key2', true)
    value = @@reg.readBoolEntry('Settings', 'bool_key2')
    assert_equal(true, value)

    @@reg.writeBoolEntry('Settings', 'bool_key2', false)
    value = @@reg.readBoolEntry('Settings', 'bool_key2')
    assert_equal(false, value)
  end

  # test of writeIntEntry
  def test_write_int_entry
    @@reg.writeIntEntry('Settings', 'int_key2', 456)
    value = @@reg.readIntEntry('Settings', 'int_key2')
    assert_equal(456, value)
  end

  # test of writeRealEntry
  def test_write_real_entry
    @@reg.writeRealEntry('Settings', 'real_key2', 25.5)
    value = @@reg.readRealEntry('Settings', 'real_key2')
    assert_equal(25.5, value)
  end

  # test of writeStringEntry
  def test_write_string_entry
    @@reg.writeStringEntry('Settings', 'string_key2', 'test value')
    string = @@reg.readStringEntry('Settings', 'string_key2')
    assert_equal('test value', string)
  end

  # test of deleteEntry
  def test_delete_entry
    @@reg.writeStringEntry('Settings', 'key1', 'dummy value')
    string = @@reg.readStringEntry('Settings', 'key1')
    assert_equal('dummy value', string)
    @@reg.writeStringEntry('Settings', 'key2', 'dummy value')
    string = @@reg.readStringEntry('Settings', 'key2')
    assert_equal('dummy value', string)
    @@reg.deleteEntry('Settings', 'key1')
    string = @@reg.readStringEntry('Settings', 'key1', 'no value')
    assert_equal('no value', string)

    # test of deleteEntry when key doesn't exist
    @@reg.deleteEntry('Settings', 'key1')
    string = @@reg.readStringEntry('Settings', 'key1', 'no value')
    assert_equal('no value', string)

    # test of deleteEntry when section doesn't exist
    @@reg.deleteSection('Settings')
    @@reg.deleteEntry('Settings', 'key2')
    string = @@reg.readStringEntry('Settings', 'key2', 'no value')
    assert_equal('no value', string)
  end

  # test of deleteSection
  def test_delete_section
    @@reg.writeStringEntry('Settings', 'key', 'dummy value')
    @@reg.deleteSection('Settings')
    string = @@reg.readStringEntry('Settings', 'key', 'no value')
    assert_equal('no value', string)

    # test of deleteSection when section doesn't exist
    @@reg.deleteSection('Settings')
    string = @@reg.readStringEntry('Settings', 'key', 'no value')
    assert_equal('no value', string)
  end

  # test of appKey
  def test_app_key
    assert_equal(@@app_key, @@reg.appKey)
  end

  # test of vendorKey
  def test_vendor_key
    assert_equal(@@vendor_key, @@reg.vendorKey)
  end
end
